
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_ENTITY_HH_
#define _MOOF_ENTITY_HH_

#include <boost/shared_ptr.hpp>

#include <Moof/Aabb.hh>
#include <Moof/Cullable.hh>
#include <Moof/Drawable.hh>
#include <Moof/Sphere.hh>


namespace Mf {


class Entity;
typedef boost::shared_ptr<Entity> EntityP;

class Frustum;


/**
 * Interface for game objects that can be drawn to the screen and have a
 * specified size.
 */

class Entity : public Cullable, public Drawable
{
public:
	virtual ~Entity() {}

	const Aabb& getAabb() const
	{
		return aabb_;
	}

	const Sphere& getSphere() const
	{
		return sphere_;
	}

	void drawIfVisible(Scalar alpha, const Frustum& frustum) const
	{
		if (isVisible(frustum)) draw(alpha);
	}

protected:
	Aabb	aabb_;
	Sphere	sphere_;
};


} // namespace Mf

#endif // _MOOF_ENTITY_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

